package nl.amis.view;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.el.ELContext;
import javax.el.ExpressionFactory;

import javax.faces.application.Application;
import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;

import oracle.adf.view.rich.component.rich.data.RichColumn;
import oracle.adf.view.rich.component.rich.input.RichInputText;
import oracle.adf.view.rich.component.rich.layout.RichPanelFormLayout;
import oracle.adf.view.rich.component.rich.layout.RichPanelGroupLayout;
import oracle.adf.view.rich.component.rich.layout.RichPanelHeader;
import oracle.adf.view.rich.component.rich.layout.RichPanelLabelAndMessage;
import oracle.adf.view.rich.component.rich.nav.RichCommandButton;
import oracle.adf.view.rich.component.rich.nav.RichCommandImageLink;
import oracle.adf.view.rich.component.rich.output.RichContextInfo;
import oracle.adf.view.rich.component.rich.output.RichOutputText;
import oracle.adf.view.rich.event.ClientListenerSet;

import org.apache.myfaces.trinidad.component.UIXEditableValue;
import org.apache.myfaces.trinidad.component.UIXEditableValue;

public class PageProcessorForTextEditing {

    private static final String ICON_FOR_OPENING_COMPONENT_TEXTRESOURCE_EDITOR =
        "/images/pencil.png";

    private static final String RICHCOMMANDIMAGELINK_FOR_TEXTEDITICON_ID_POSTFIX =
        "textEditorIcon";
    private Boolean textResourceEditingMode;

    private final String EDIT_COMPONENT_TEXTRESOURCES_POPUP =
        "texteditPopup:editTextResources";

    public void enterTextResourceEditingMode(ActionEvent actionEvent) {
        if (gettextResourceEditingMode() == null ||
            !gettextResourceEditingMode()) {
            settextResourceEditingMode(true);
            alreadyProcessed = false;
        } else {
            settextResourceEditingMode(false);
        }
    }

    public void settextResourceEditingMode(Boolean textResourceEditingMode) {
        this.textResourceEditingMode = textResourceEditingMode;
    }

    public Boolean gettextResourceEditingMode() {
        return textResourceEditingMode;
    }

    boolean alreadyProcessed = false;

    public void processPage() {
        if (textResourceEditingMode != null && textResourceEditingMode) {
            if (!alreadyProcessed) {
                alreadyProcessed = true;
                UIViewRoot root =
                    FacesContext.getCurrentInstance().getViewRoot();
                processUIChildren(root);
            }
        }
    }

    public void processUIChildren(UIComponent component) {
        if (component instanceof RichInputText) {
            addContextFacet(component);
        } else if (component instanceof RichCommandButton) {
            wrapComponentInPGLandAddCommandImageLink(component);
        } else {
            addPopupBehaviorOnUIComponent(component);
        }
    

    UIComponent kid;
    
        for (Iterator kids = component.getFacetsAndChildren(); kids.hasNext();
        ) {
            kid = (UIComponent)kids.next();
            // do not process the text editor popup and its children
            if (!("editTextResources".equalsIgnoreCase(kid.getId()))) {
                processUIChildren(kid);
            }
        }
    }

    private void addPopupBehaviorOnUIComponent(UIComponent component) {
        if (componentIsTextEditable(component)) {
            if (component instanceof RichOutputText) {
              ((RichOutputText)component).setClientComponent(true);
            }
            
            ClientListenerSet cls = findClientListenerSet(component);
            if (cls != null) {
                String triggerType = "mouseOver";
                if (component instanceof RichPanelHeader) {
                    triggerType = "dblClick";
                }
                cls.addBehavior("new AdfShowPopupBehavior(':::" +
                                EDIT_COMPONENT_TEXTRESOURCES_POPUP +
                                "','','','" + triggerType + "')");
            }
        }
    }


    private ClientListenerSet findClientListenerSet(UIComponent uiComponent) {
        ClientListenerSet cls = null;
        boolean found = false;
        // use reflection to invoke the getClientListeners method if it exists on this uiComponent
        Method[] allMethods = uiComponent.getClass().getDeclaredMethods();
        for (Method m : allMethods) {
            String mname = m.getName();
            if (mname.startsWith("getClientListeners")) {
                try {
                    m.setAccessible(true);
                    Object o = m.invoke(uiComponent);
                    if (o instanceof ClientListenerSet) {
                        cls = (ClientListenerSet)o;
                        // make current clientlistenerset null
                        // this removes all client listeners!
                        cls = null;
                        //return cls;
                    }
                    found = true;
                    break;
                    // Handle any exceptions thrown by method to be invoked.
                } catch (InvocationTargetException x) {
                    Throwable cause = x.getCause();

                } catch (IllegalAccessException e) {
                }
            }
        }
        if (cls == null && found) {
            cls = new ClientListenerSet();
            for (Method m : allMethods) {
                String mname = m.getName();
                if (mname.startsWith("setClientListeners")) {
                    try {
                        Object o = m.invoke(uiComponent, cls);
                        // Handle any exceptions thrown by method to be invoked.
                    } catch (InvocationTargetException x) {
                        Throwable cause = x.getCause();

                    } catch (IllegalAccessException e) {
                    }
                }
            }
        }
        return cls;
    }


    private void addContextFacet(UIComponent uiComponent) {
        RichContextInfo rci = new RichContextInfo();
        rci.setId(uiComponent.getId() + "Ctx");
        Set ca = null;
        if (uiComponent.getFacets().containsKey("context")) {
            rci = (RichContextInfo)uiComponent.getFacets().get("context");
            ca = rci.getClientAttributes();
            rci.getAttributes().put("componentId", uiComponent.getId());
        } else {
            uiComponent.getFacets().put("context", rci);
        }
        ClientListenerSet set = rci.getClientListeners();
        if (set == null) {
            set = new ClientListenerSet();
            rci.setClientListeners(set);
            set.addBehavior("new AdfShowPopupBehavior('" +
                            EDIT_COMPONENT_TEXTRESOURCES_POPUP +
                            "',AdfRichPopup.ALIGN_BEFORE_START,'" +
                            uiComponent.getId() + "','contextInfo')");
        }
    }

    // this method takes a component
    // it creates a RichPanelGroupLayout, moves the component into that RPGL and adds
    // a commandImageLink that will open the text editor for the component
    // the RPGL is then set as the new child for the component's parent
    // before: parent component => Component
    // after:  parent component => RPGL -> Component
    //                                  -> RichCommandImageLink -> ShowPopupBehavior (=> textResourceEditorPopup )

    private void wrapComponentInPGLandAddCommandImageLink(UIComponent component) {
        UIComponent parent = component.getParent();
        if (parent instanceof RichPanelGroupLayout &&
            component.getId().endsWith("txtpgl")) { // already wrapped
        } else {
            int i = parent.getChildren().indexOf(component);
            RichPanelGroupLayout rp = new RichPanelGroupLayout();
            rp.setId(component.getId());
            rp.setLayout(RichPanelGroupLayout.LAYOUT_HORIZONTAL);
            component.setId(component.getId() + "txtpgl");
            if (i == -1) {
                // component is in parent's facet
                Iterator<String> iterator =
                    parent.getFacets().keySet().iterator();
                while (iterator.hasNext()) {
                    String facet = iterator.next();
                    if (component == parent.getFacet(facet)) {
                        parent.getFacets().put(facet, rp);
                        break;
                    }
                }
            } else {
                parent.getChildren().set(i, rp);
            }
            rp.getChildren().add(component);
            component.setParent(rp);
            RichCommandImageLink rcil = new RichCommandImageLink();
            rcil.setId(component.getId() +
                       RICHCOMMANDIMAGELINK_FOR_TEXTEDITICON_ID_POSTFIX);
            rcil.setIcon(ICON_FOR_OPENING_COMPONENT_TEXTRESOURCE_EDITOR);
            rcil.setShortDesc("Edit text resources");
            ClientListenerSet set = new ClientListenerSet();
            rcil.setClientListeners(set);
            set.addBehavior("new AdfShowPopupBehavior(':::" +
                            EDIT_COMPONENT_TEXTRESOURCES_POPUP +
                            "','','','action')");
            java.util.Set<String> clientAttributes =
                new java.util.HashSet<String>(3);
            rcil.setClientAttributes(clientAttributes);
            clientAttributes.add("componentId");
            rcil.getAttributes().put("componentId", component.getId());
            rp.getChildren().add(rcil);
        }
    }


    private boolean componentIsTextEditable(UIComponent component) {
        boolean result = false;
        if (component instanceof UIXEditableValue ||
            component instanceof RichOutputText ||
            component instanceof RichCommandButton ||
            component instanceof RichPanelHeader ||
            component instanceof RichPanelLabelAndMessage) {
            result = true;
        }
        return result;
    }
}
